@@ -1,13 +1,17 @@
 //! Multiple precision integer
 
 use crate::{
-    base64::{self, Decode},
+    decoder::{Decode, Decoder},
+    encoder::{Encode, Encoder},
     Error, Result,
 };
 use alloc::vec::Vec;
 use core::fmt;
 use zeroize::Zeroize;
 
+#[cfg(feature = "subtle")]
+use subtle::{Choice, ConstantTimeEq};
+
 /// Multiple precision integer, a.k.a. "mpint".
 ///
 /// This type is used for representing the big integer components of
@@ -84,11 +88,21 @@ impl AsRef<[u8]> for MPInt {
 }
 
 impl Decode for MPInt {
-    fn decode(decoder: &mut base64::Decoder<'_>) -> Result<Self> {
+    fn decode(decoder: &mut impl Decoder) -> Result<Self> {
         decoder.decode_byte_vec()?.try_into()
     }
 }
 
+impl Encode for MPInt {
+    fn encoded_len(&self) -> Result<usize> {
+        Ok(4 + self.as_bytes().len())
+    }
+
+    fn encode(&self, encoder: &mut impl Encoder) -> Result<()> {
+        encoder.encode_byte_slice(self.as_bytes())
+    }
+}
+
 impl TryFrom<&[u8]> for MPInt {
     type Error = Error;
 
@@ -147,6 +161,14 @@ impl fmt::UpperHex for MPInt {
     }
 }
 
+#[cfg(feature = "subtle")]
+#[cfg_attr(docsrs, doc(cfg(feature = "subtle")))]
+impl ConstantTimeEq for MPInt {
+    fn ct_eq(&self, other: &Self) -> Choice {
+        self.as_ref().ct_eq(other.as_ref())
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::MPInt;
